@@ -1,5 +1,6 @@
 import os
 import time
+import traceback
 import uuid
 import warnings
 from typing import Dict, Iterable, List, Mapping, Optional, Union
@@ -9,10 +10,10 @@ import openai
 import openai.types as openai_types
 import openai.types.chat as openai_chat_types
 
-import maitai
 from maitai._config import config
 from maitai._evaluator import Evaluator
 from maitai._inference import (
+    Inference,
     InferenceException,
     InferenceWarning,
     MaitaiConnectionError,
@@ -20,6 +21,7 @@ from maitai._inference import (
 from maitai._types import (
     Body,
     EvaluateCallback,
+    FallbackConfig,
     Headers,
     MaitaiChunk,
     MaitaiCompletion,
@@ -27,17 +29,22 @@ from maitai._types import (
     ToolFunction,
 )
 from maitai._utils import (
-    OpenAISchemaGenerator,
     convert_open_ai_chat_completion_chunk,
     convert_openai_chat_completion,
     get_chat_completion_params,
     required_args,
+    type_to_response_format_param,
 )
+from maitai.models.chat import (
+    AzureParams,
+    ChatCompletionParams,
+    ClientParams,
+    EvaluationContentType,
+)
+from maitai.models.config import InferenceLocations
+from maitai.models.inference import InferenceStreamResponse
+from maitai.models.metric import RequestTimingMetric
 from maitai.tools import Tools
-from maitai_gen.chat import ChatCompletionParams, ClientParams, EvaluationContentType
-from maitai_gen.config import InferenceLocations
-from maitai_gen.inference import InferenceStreamResponse
-from maitai_gen.metric import RequestTimingMetric
 
 DEFAULT_MAX_RETRIES = 2
 
@@ -52,6 +59,11 @@ class Maitai:
         groq_api_key: Optional[str] = None,
         anthropic_api_key: Optional[str] = None,
         cerebras_api_key: Optional[str] = None,
+        azure_api_key: Optional[str] = None,
+        sambanova_api_key: Optional[str] = None,
+        azure_deployment: Optional[str] = None,
+        azure_api_version: Optional[str] = None,
+        azure_endpoint: Optional[str] = None,
         organization: Optional[str] = None,
         project: Optional[str] = None,
         base_url: Union[str, httpx.URL, None] = None,
@@ -65,20 +77,15 @@ class Maitai:
         if not maitai_api_key:
             maitai_api_key = os.environ.get("MAITAI_API_KEY")
         if maitai_api_key:
-            config.initialize(maitai_api_key)
+            config.initialize(maitai_api_key, use_async=False)
         else:
             raise ValueError(
                 "MAITAI_API_KEY has not been set. Either set via environment variable or pass it directly into the client."
             )
         if api_key:
-            if not openai_api_key:
-                openai_api_key = api_key
-            if not groq_api_key:
-                groq_api_key = api_key
-            if not anthropic_api_key:
-                anthropic_api_key = api_key
-            if not cerebras_api_key:
-                cerebras_api_key = api_key
+            raise ValueError(
+                "api_key is deprecated. Use provider specific keys instead. Supported variables: openai_api_key, groq_api_key, anthropic_api_key, cerebras_api_key, azure_api_key, sambanova_api_key"
+            )
         if openai_api_key:
             config.auth_keys.openai_api_key.key_value = openai_api_key
         if groq_api_key:
@@ -87,10 +94,36 @@ class Maitai:
             config.auth_keys.anthropic_api_key.key_value = anthropic_api_key
         if cerebras_api_key:
             config.auth_keys.cerebras_api_key.key_value = cerebras_api_key
+        if azure_api_key:
+            config.auth_keys.azure_api_key.key_value = azure_api_key
+        if sambanova_api_key:
+            config.auth_keys.sambanova_api_key.key_value = sambanova_api_key
         self._client = None
-        if api_key or openai_api_key or os.environ.get("OPENAI_API_KEY"):
+        azure_params = None
+        if azure_api_key:
+            azure_params = AzureParams(
+                endpoint=azure_endpoint,
+                deployment=azure_deployment,
+                api_version=azure_api_version,
+            )
+            self._client = openai.AzureOpenAI(
+                azure_endpoint=azure_endpoint,
+                azure_deployment=azure_deployment,
+                api_key=azure_api_key,
+                api_version=azure_api_version,
+                organization=organization,
+                project=project,
+                base_url=base_url,
+                timeout=timeout,
+                max_retries=max_retries,
+                default_headers=default_headers,
+                default_query=default_query,
+                http_client=http_client,
+                _strict_response_validation=_strict_response_validation,
+            )
+        elif openai_api_key or os.environ.get("OPENAI_API_KEY"):
             self._client = openai.OpenAI(
-                api_key=api_key or openai_api_key or os.environ.get("OPENAI_API_KEY"),
+                api_key=openai_api_key or os.environ.get("OPENAI_API_KEY"),
                 organization=organization,
                 project=project,
                 base_url=base_url,
@@ -107,6 +140,7 @@ class Maitai:
                 base_url=base_url or "",
                 default_headers=default_headers or {},
                 default_query=default_query or {},
+                azure_params=azure_params,
             )
         self.chat = Chat(self._client, client_params)
 
@@ -124,6 +158,8 @@ class Completions:
     ):
         self._client = client
         self._client_params = client_params or ClientParams()
+        self.evaluator = Evaluator()
+        self.inference = Inference()
 
     @required_args(
         ["intent", "application", "messages"],
@@ -155,6 +191,7 @@ class Completions:
         assistant: bool = False,
         metadata: dict = {},
         safe_mode: bool = None,
+        fallback_config: FallbackConfig = None,
         # OpenAI Arguments
         messages: Iterable[openai_chat_types.ChatCompletionMessageParam],
         model: Union[str, openai_types.ChatModel, openai.NotGiven] = openai.NOT_GIVEN,
@@ -270,8 +307,25 @@ class Completions:
             timeout = maitai_config.timeout
         if context_retrieval_enabled is None:
             context_retrieval_enabled = maitai_config.context_retrieval_enabled
-        if fallback_model is None and maitai_config.fallback_model:
-            fallback_model = maitai_config.fallback_model
+        if fallback_model:
+            warnings.warn(
+                "fallback_model is deprecated. Use fallback_config instead.",
+                DeprecationWarning,
+            )
+        if fallback_config is None and maitai_config.fallback_config:
+            fallback_config = {
+                "model": maitai_config.fallback_config.model,
+                "strategy": maitai_config.fallback_config.strategy,
+                "timeout": maitai_config.fallback_config.timeout,
+            }
+        if fallback_config is None and fallback_model:
+            fallback_config = {
+                "model": fallback_model,
+                "strategy": "reactive",
+                "timeout": None,
+            }
+        if fallback_config and "model" in fallback_config:
+            fallback_model = fallback_config["model"]
         completion_params = get_chat_completion_params(
             messages=messages,
             model=model,
@@ -314,17 +368,18 @@ class Completions:
                 "fallback_model": fallback_model,
                 "user_id": user_id,
                 "assistant": assistant,
-                "auth_keys": config.auth_keys.to_dict(),
+                "auth_keys": config.auth_keys.model_dump(),
                 "metadata": metadata,
                 "safe_mode": safe_mode,
+                "fallback_config": fallback_config,
             }
 
             if self._client_params is not None:
-                request_dict["client_params"] = self._client_params.to_dict()
+                request_dict["client_params"] = self._client_params.model_dump()
 
             try:
                 timing.time_pre_request = time.time()
-                response = maitai.Inference.infer(
+                response = self.inference.infer(
                     request_dict, callback, response_timeout
                 )
                 if stream:
@@ -333,10 +388,9 @@ class Completions:
                     retry_args["server_side_inference"] = False
                     retry_args["model"] = model
                     retry_args["fallback_model"] = fallback_model
-                    return _process_inference_stream(
+                    return self._process_inference_stream(
                         response,
                         retry_args,
-                        self.create,
                         timing,
                         includes_usage=(
                             stream_options.get("include_usage", False)
@@ -354,10 +408,12 @@ class Completions:
                         raise InferenceException(resp.error)
                     timing.request_id = resp.chat_completion_response.request_id
                     timing.time_request_end = time.time()
-                    maitai.Inference.store_request_timing_data(timing)
-                    return MaitaiCompletion(
-                        resp.chat_completion_response, response_format=response_format
+                    self.inference.store_request_timing_data(timing)
+                    completion = MaitaiCompletion.model_validate(
+                        resp.chat_completion_response.model_dump()
                     )
+                    completion.set_response_format(response_format)
+                    return completion
                 raise InferenceException("No data received")
             except MaitaiConnectionError as e:
                 if "gpt" not in model and fallback_model and "gpt" in fallback_model:
@@ -366,9 +422,12 @@ class Completions:
                     raise e
                 if safe_mode:
                     raise e
+                traceback.print_exc()
                 print("Maitai issue, retrying on client side")
                 server_side_inference = False
         if not server_side_inference:
+            if "openai:" in model:
+                model = model.replace("openai:", "")
             if self._client is None:
                 self._client = openai.OpenAI(
                     default_query=self._client_params.default_query or None,
@@ -378,9 +437,7 @@ class Completions:
             if hasattr(response_format, "model_json_schema"):
                 response_format = {
                     "type": "json_schema",
-                    "json_schema": response_format.model_json_schema(
-                        schema_generator=OpenAISchemaGenerator, mode="serialization"
-                    ),
+                    "json_schema": type_to_response_format_param(response_format),
                 }
             response = self._client.chat.completions.create(
                 messages=messages,
@@ -411,7 +468,7 @@ class Completions:
                 parallel_tool_calls=parallel_tool_calls,
             )
             if stream:
-                return _process_openai_stream(
+                return self._process_openai_stream(
                     session_id,
                     reference_id,
                     intent,
@@ -434,7 +491,7 @@ class Completions:
                     }
                 ]
                 if evaluation_enabled:
-                    maitai.Evaluator.evaluate(
+                    self.evaluator.evaluate(
                         session_id=session_id,
                         reference_id=reference_id,
                         intent=intent,
@@ -448,7 +505,7 @@ class Completions:
                         metadata=metadata,
                     )
                 else:
-                    maitai.Inference.store_chat_response(
+                    self.inference.store_chat_response(
                         session_id=session_id,
                         reference_id=reference_id,
                         intent=intent,
@@ -460,110 +517,112 @@ class Completions:
                         timing=timing,
                         metadata=metadata,
                     )
-                return MaitaiCompletion(
-                    maitai_completion, response_format=response_format
+                completion = MaitaiCompletion.model_validate(
+                    maitai_completion.model_dump()
                 )
+                completion.set_response_format(response_format)
+                return completion
 
-
-def _process_openai_stream(
-    session_id: Union[str, int],
-    reference_id: Union[str, int, None],
-    intent: str,
-    application_ref_name: str,
-    messages: Iterable[openai_chat_types.ChatCompletionMessageParam],
-    stream: openai.Stream[openai_chat_types.ChatCompletionChunk],
-    evaluation_enabled: bool,
-    chat_completion_params: ChatCompletionParams,
-    timing: RequestTimingMetric,
-    callback: Optional[EvaluateCallback] = None,
-    metadata: dict = {},
-) -> Iterable[MaitaiChunk]:
-    full_body = ""
-    last_chunk = None
-    for chunk in stream:
-        if last_chunk is None:
-            timing.time_first_chunk = time.time()
-        maitai_chunk = convert_open_ai_chat_completion_chunk(chunk)
-        if maitai_chunk.choices:
-            last_chunk = maitai_chunk
-            content = maitai_chunk.choices[0].delta.content
-            if content is not None:
-                full_body += content
-        if maitai_chunk.usage and last_chunk is not None and not last_chunk.usage:
-            last_chunk.usage = maitai_chunk.usage
-        yield MaitaiChunk(maitai_chunk)
-    timing.time_request_end = time.time()
-    if evaluation_enabled:
-        full_conv = messages + [{"role": "assistant", "content": full_body}]
-        Evaluator.evaluate(
-            session_id=session_id,
-            reference_id=reference_id,
-            intent=intent,
-            content_type=EvaluationContentType.MESSAGE,
-            content=full_conv,
-            application_ref_name=application_ref_name,
-            callback=callback,
-            chat_completion_chunk=last_chunk,
-            completion_params=chat_completion_params,
-            timing=timing,
-            metadata=metadata,
-        )
-    else:
-        maitai.Inference.store_chat_response(
-            session_id=session_id,
-            reference_id=reference_id,
-            intent=intent,
-            application_ref_name=application_ref_name,
-            completion_params=chat_completion_params,
-            final_chunk=last_chunk,
-            content=full_body,
-            chat_completion_response=None,
-            timing=timing,
-            metadata=metadata,
-        )
-
-
-def _process_inference_stream(
-    stream: Iterable[InferenceStreamResponse],
-    retry_args: dict,
-    retry_func,
-    timing: RequestTimingMetric,
-    includes_usage: bool = False,
-) -> Iterable[MaitaiChunk]:
-    first = True
-    try:
-        for infer_resp in stream:
-            if infer_resp.warning:
-                warnings.warn(infer_resp.warning, InferenceWarning)
-            if infer_resp.error:
-                raise InferenceException(infer_resp.error)
-            chunk = infer_resp.chat_completion_chunk
-            if chunk is not None:
-                if first:
-                    timing.time_first_chunk = time.time()
-                    timing.request_id = chunk.request_id
-                    first = False
-                yield MaitaiChunk(chunk)
-                if chunk.choices and chunk.choices[0].finish_reason:
-                    timing.time_request_end = time.time()
-                    maitai.Inference.store_request_timing_data(timing)
-                if chunk.usage:
-                    return
-                if (
-                    not includes_usage
-                    and chunk.choices
-                    and chunk.choices[0].finish_reason
-                ):
-                    return
-    except MaitaiConnectionError as e:
-        if "gpt" not in retry_args.get("model", "") and "gpt" in retry_args.get(
-            "fallback_model", ""
-        ):
-            retry_args["model"] = retry_args["fallback_model"]
-        if "gpt" in retry_args.get("model", ""):
-            print("Maitai issue, retrying on client side")
-            response = retry_func(**retry_args)
-            for resp in response:
-                yield resp
+    def _process_openai_stream(
+        self,
+        session_id: Union[str, int],
+        reference_id: Union[str, int, None],
+        intent: str,
+        application_ref_name: str,
+        messages: Iterable[openai_chat_types.ChatCompletionMessageParam],
+        stream: openai.Stream[openai_chat_types.ChatCompletionChunk],
+        evaluation_enabled: bool,
+        chat_completion_params: ChatCompletionParams,
+        timing: RequestTimingMetric,
+        callback: Optional[EvaluateCallback] = None,
+        metadata: dict = {},
+    ) -> Iterable[MaitaiChunk]:
+        full_body = ""
+        last_chunk = None
+        for chunk in stream:
+            if last_chunk is None:
+                timing.time_first_chunk = time.time()
+            maitai_chunk = convert_open_ai_chat_completion_chunk(chunk)
+            if maitai_chunk.choices:
+                last_chunk = maitai_chunk
+                content = maitai_chunk.choices[0].delta.content
+                if content is not None:
+                    full_body += content
+            if maitai_chunk.usage and last_chunk is not None and not last_chunk.usage:
+                last_chunk.usage = maitai_chunk.usage
+            yield MaitaiChunk.model_validate(maitai_chunk.model_dump())
+        timing.time_request_end = time.time()
+        if evaluation_enabled:
+            full_conv = messages + [{"role": "assistant", "content": full_body}]
+            self.evaluator.evaluate(
+                session_id=session_id,
+                reference_id=reference_id,
+                intent=intent,
+                content_type=EvaluationContentType.MESSAGE,
+                content=full_conv,
+                application_ref_name=application_ref_name,
+                callback=callback,
+                chat_completion_chunk=last_chunk,
+                completion_params=chat_completion_params,
+                timing=timing,
+                metadata=metadata,
+            )
         else:
-            raise e
+            self.inference.store_chat_response(
+                session_id=session_id,
+                reference_id=reference_id,
+                intent=intent,
+                application_ref_name=application_ref_name,
+                completion_params=chat_completion_params,
+                final_chunk=last_chunk,
+                content=full_body,
+                chat_completion_response=None,
+                timing=timing,
+                metadata=metadata,
+            )
+
+    def _process_inference_stream(
+        self,
+        stream: Iterable[InferenceStreamResponse],
+        retry_args: dict,
+        timing: RequestTimingMetric,
+        includes_usage: bool = False,
+    ) -> Iterable[MaitaiChunk]:
+        first = True
+        try:
+            for infer_resp in stream:
+                if infer_resp.warning:
+                    warnings.warn(infer_resp.warning, InferenceWarning)
+                if infer_resp.error:
+                    raise InferenceException(infer_resp.error)
+                chunk = infer_resp.chat_completion_chunk
+                if chunk is not None:
+                    if first:
+                        timing.time_first_chunk = time.time()
+                        timing.request_id = chunk.request_id
+                        first = False
+                    yield MaitaiChunk.model_validate(chunk.model_dump())
+                    if chunk.choices and chunk.choices[0].finish_reason:
+                        timing.time_request_end = time.time()
+                        self.inference.store_request_timing_data(timing)
+                    if chunk.usage:
+                        return
+                    if (
+                        not includes_usage
+                        and chunk.choices
+                        and chunk.choices[0].finish_reason
+                    ):
+                        return
+        except MaitaiConnectionError as e:
+            if "gpt" not in retry_args.get("model", "") and "gpt" in (
+                retry_args.get("fallback_model", "") or ""
+            ):
+                retry_args["model"] = retry_args["fallback_model"]
+            if "gpt" in retry_args.get("model", ""):
+                print("Maitai issue, retrying on client side")
+                retry_args["model"] = retry_args["model"].replace("openai:", "")
+                response = self.create(**retry_args)
+                for resp in response:
+                    yield resp
+            else:
+                raise e
