@@ -1,3 +1,4 @@
+import asyncio
 import os
 import time
 import uuid
@@ -9,10 +10,10 @@ import openai
 import openai.types as openai_types
 import openai.types.chat as openai_chat_types
 
-import maitai
 from maitai._config import config
 from maitai._evaluator import Evaluator
 from maitai._inference import (
+    Inference,
     InferenceException,
     InferenceWarning,
     MaitaiConnectionError,
@@ -58,6 +59,7 @@ class MaitaiAsync:
         groq_api_key: Optional[str] = None,
         anthropic_api_key: Optional[str] = None,
         cerebras_api_key: Optional[str] = None,
+        sambanova_api_key: Optional[str] = None,
         azure_api_key: Optional[str] = None,
         azure_deployment: Optional[str] = None,
         azure_api_version: Optional[str] = None,
@@ -82,7 +84,7 @@ class MaitaiAsync:
             )
         if api_key:
             raise ValueError(
-                "api_key is deprecated. Use provider specific keys instead. Supported variables: openai_api_key, groq_api_key, anthropic_api_key, cerebras_api_key, azure_api_key"
+                "api_key is deprecated. Use provider specific keys instead. Supported variables: openai_api_key, groq_api_key, anthropic_api_key, cerebras_api_key, azure_api_key, sambanova_api_key"
             )
         if openai_api_key:
             config.auth_keys.openai_api_key.key_value = openai_api_key
@@ -94,6 +96,8 @@ class MaitaiAsync:
             config.auth_keys.cerebras_api_key.key_value = cerebras_api_key
         if azure_api_key:
             config.auth_keys.azure_api_key.key_value = azure_api_key
+        if sambanova_api_key:
+            config.auth_keys.sambanova_api_key.key_value = sambanova_api_key
         self._client = None
         if azure_api_key:
             azure_params = AzureParams(
@@ -142,7 +146,11 @@ class MaitaiAsync:
 
 
 class AsyncChat:
-    def __init__(self, client=None, client_params: ClientParams = None):
+    def __init__(
+        self,
+        client=None,
+        client_params: ClientParams = None,
+    ):
         self.completions = AsyncCompletions(client, client_params)
 
 
@@ -154,6 +162,8 @@ class AsyncCompletions:
     ):
         self._client = client
         self._client_params = client_params or ClientParams()
+        self.inference_client = Inference()
+        self.evaluator = Evaluator()
 
     @required_args(
         ["intent", "application", "messages"],
@@ -370,7 +380,7 @@ class AsyncCompletions:
 
             try:
                 timing.time_pre_request = time.time()
-                response = await maitai.Inference.infer_async(
+                response = await self.inference_client.infer_async(
                     request_dict, callback, response_timeout
                 )
                 if stream:
@@ -379,10 +389,9 @@ class AsyncCompletions:
                     retry_args["server_side_inference"] = False
                     retry_args["model"] = model
                     retry_args["fallback_model"] = fallback_model
-                    return _process_inference_stream_async(
+                    return self._process_inference_stream_async(
                         response,
                         retry_args,
-                        self.create,
                         timing,
                         includes_usage=(
                             stream_options.get("include_usage", False)
@@ -400,7 +409,9 @@ class AsyncCompletions:
                         raise InferenceException(resp.error)
                     timing.request_id = resp.chat_completion_response.request_id
                     timing.time_request_end = time.time()
-                    maitai.Inference.store_request_timing_data(timing)
+                    asyncio.create_task(
+                        self.inference_client.send_request_timing_data_async(timing)
+                    )
                     completion = MaitaiCompletion.model_validate(
                         resp.chat_completion_response.model_dump()
                     )
@@ -417,6 +428,8 @@ class AsyncCompletions:
                 print("Maitai issue, retrying on client side")
                 server_side_inference = False
         if not server_side_inference:
+            if "openai:" in model:
+                model = model.replace("openai:", "")
             if self._client is None:
                 self._client = openai.AsyncOpenAI(
                     default_headers=self._client_params.default_headers or None,
@@ -457,7 +470,7 @@ class AsyncCompletions:
                 parallel_tool_calls=parallel_tool_calls,
             )
             if stream:
-                return _process_async_openai_stream(
+                return self._process_async_openai_stream(
                     session_id,
                     reference_id,
                     intent,
@@ -480,7 +493,7 @@ class AsyncCompletions:
                     }
                 ]
                 if evaluation_enabled:
-                    await maitai.Evaluator.evaluate_async(
+                    await self.evaluator.evaluate_async(
                         session_id=session_id,
                         reference_id=reference_id,
                         intent=intent,
@@ -494,7 +507,7 @@ class AsyncCompletions:
                         metadata=metadata,
                     )
                 else:
-                    maitai.Inference.store_chat_response(
+                    self.inference_client.store_chat_response_async(
                         session_id=session_id,
                         reference_id=reference_id,
                         intent=intent,
@@ -512,108 +525,110 @@ class AsyncCompletions:
                 completion.set_response_format(response_format)
                 return completion
 
-
-async def _process_async_openai_stream(
-    session_id: Union[str, int],
-    reference_id: Union[str, int, None],
-    intent: str,
-    application_ref_name: str,
-    messages: Iterable[openai_chat_types.ChatCompletionMessageParam],
-    stream: openai.AsyncStream[openai_chat_types.ChatCompletionChunk],
-    evaluation_enabled: bool,
-    chat_completion_params: ChatCompletionParams,
-    timing: RequestTimingMetric,
-    callback: Optional[EvaluateCallback] = None,
-    metadata: dict = {},
-) -> AsyncIterable[MaitaiChunk]:
-    full_body = ""
-    last_chunk = None
-    async for chunk in stream:
+    async def _process_async_openai_stream(
+        self,
+        session_id: Union[str, int],
+        reference_id: Union[str, int, None],
+        intent: str,
+        application_ref_name: str,
+        messages: Iterable[openai_chat_types.ChatCompletionMessageParam],
+        stream: openai.AsyncStream[openai_chat_types.ChatCompletionChunk],
+        evaluation_enabled: bool,
+        chat_completion_params: ChatCompletionParams,
+        timing: RequestTimingMetric,
+        callback: Optional[EvaluateCallback] = None,
+        metadata: dict = {},
+    ) -> AsyncIterable[MaitaiChunk]:
+        full_body = ""
+        last_chunk = None
+        async for chunk in stream:
+            if last_chunk is None:
+                timing.time_first_chunk = time.time()
+            maitai_chunk = convert_open_ai_chat_completion_chunk(chunk)
+            if maitai_chunk.choices:
+                last_chunk = maitai_chunk
+                content = maitai_chunk.choices[0].delta.content
+                if content is not None:
+                    full_body += content
+            if maitai_chunk.usage and last_chunk is not None and not last_chunk.usage:
+                last_chunk.usage = maitai_chunk.usage
+            yield MaitaiChunk.model_validate(maitai_chunk.model_dump())
+        timing.time_request_end = time.time()
         if last_chunk is None:
-            timing.time_first_chunk = time.time()
-        maitai_chunk = convert_open_ai_chat_completion_chunk(chunk)
-        if maitai_chunk.choices:
-            last_chunk = maitai_chunk
-            content = maitai_chunk.choices[0].delta.content
-            if content is not None:
-                full_body += content
-        if maitai_chunk.usage and last_chunk is not None and not last_chunk.usage:
-            last_chunk.usage = maitai_chunk.usage
-        yield MaitaiChunk.model_validate(maitai_chunk.model_dump())
-    timing.time_request_end = time.time()
-    if last_chunk is None:
-        return
-    if evaluation_enabled:
-        full_conv = messages + [{"role": "assistant", "content": full_body}]
-        await Evaluator.evaluate_async(
-            session_id=session_id,
-            reference_id=reference_id,
-            intent=intent,
-            content_type=EvaluationContentType.MESSAGE,
-            content=full_conv,
-            application_ref_name=application_ref_name,
-            callback=callback,
-            chat_completion_chunk=last_chunk,
-            completion_params=chat_completion_params,
-            timing=timing,
-            metadata=metadata,
-        )
-    else:
-        maitai.Inference.store_chat_response(
-            session_id=session_id,
-            reference_id=reference_id,
-            intent=intent,
-            application_ref_name=application_ref_name,
-            completion_params=chat_completion_params,
-            final_chunk=last_chunk,
-            content=full_body,
-            chat_completion_response=None,
-            timing=timing,
-            metadata=metadata,
-        )
-
-
-async def _process_inference_stream_async(
-    stream: AsyncIterable[InferenceStreamResponse],
-    retry_args: dict,
-    retry_func,
-    timing: RequestTimingMetric,
-    includes_usage: bool = False,
-) -> AsyncIterable[MaitaiChunk]:
-    first = True
-    try:
-        async for infer_resp in stream:
-            if infer_resp.warning:
-                warnings.warn(infer_resp.warning, InferenceWarning)
-            if infer_resp.error:
-                raise InferenceException(infer_resp.error)
-            chunk = infer_resp.chat_completion_chunk
-            if chunk is not None:
-                if first:
-                    timing.time_first_chunk = time.time()
-                    timing.request_id = chunk.request_id
-                    first = False
-                yield MaitaiChunk.model_validate(chunk.model_dump())
-                if chunk.choices and chunk.choices[0].finish_reason:
-                    timing.time_request_end = time.time()
-                    maitai.Inference.store_request_timing_data(timing)
-                if chunk.usage:
-                    return
-                if (
-                    not includes_usage
-                    and chunk.choices
-                    and chunk.choices[0].finish_reason
-                ):
-                    return
-    except MaitaiConnectionError as e:
-        if "gpt" not in retry_args.get("model", "") and "gpt" in (
-            retry_args.get("fallback_model", "") or ""
-        ):
-            retry_args["model"] = retry_args["fallback_model"]
-        if "gpt" in retry_args.get("model", ""):
-            print("Maitai issue, retrying on client side")
-            response = retry_func(**retry_args)
-            for resp in response:
-                yield resp
+            return
+        if evaluation_enabled:
+            full_conv = messages + [{"role": "assistant", "content": full_body}]
+            await self.evaluator.evaluate_async(
+                session_id=session_id,
+                reference_id=reference_id,
+                intent=intent,
+                content_type=EvaluationContentType.MESSAGE,
+                content=full_conv,
+                application_ref_name=application_ref_name,
+                callback=callback,
+                chat_completion_chunk=last_chunk,
+                completion_params=chat_completion_params,
+                timing=timing,
+                metadata=metadata,
+            )
         else:
-            raise e
+            self.inference_client.store_chat_response_async(
+                session_id=session_id,
+                reference_id=reference_id,
+                intent=intent,
+                application_ref_name=application_ref_name,
+                completion_params=chat_completion_params,
+                final_chunk=last_chunk,
+                content=full_body,
+                chat_completion_response=None,
+                timing=timing,
+                metadata=metadata,
+            )
+
+    async def _process_inference_stream_async(
+        self,
+        stream: AsyncIterable[InferenceStreamResponse],
+        retry_args: dict,
+        timing: RequestTimingMetric,
+        includes_usage: bool = False,
+    ) -> AsyncIterable[MaitaiChunk]:
+        first = True
+        try:
+            async for infer_resp in stream:
+                if infer_resp.warning:
+                    warnings.warn(infer_resp.warning, InferenceWarning)
+                if infer_resp.error:
+                    raise InferenceException(infer_resp.error)
+                chunk = infer_resp.chat_completion_chunk
+                if chunk is not None:
+                    if first:
+                        timing.time_first_chunk = time.time()
+                        timing.request_id = chunk.request_id
+                        first = False
+                    yield MaitaiChunk.model_validate(chunk.model_dump())
+                    if chunk.choices and chunk.choices[0].finish_reason:
+                        timing.time_request_end = time.time()
+                        asyncio.create_task(
+                            self.inference_client.send_request_timing_data_async(timing)
+                        )
+                    if chunk.usage:
+                        return
+                    if (
+                        not includes_usage
+                        and chunk.choices
+                        and chunk.choices[0].finish_reason
+                    ):
+                        return
+        except MaitaiConnectionError as e:
+            if "gpt" not in retry_args.get("model", "") and "gpt" in (
+                retry_args.get("fallback_model", "") or ""
+            ):
+                retry_args["model"] = retry_args["fallback_model"]
+            if "gpt" in retry_args.get("model", ""):
+                retry_args["model"] = retry_args["model"].replace("openai:", "")
+                print("Maitai issue, retrying on client side")
+                response = await self.create(**retry_args)
+                async for resp in response:
+                    yield resp
+            else:
+                raise e
