@@ -14,6 +14,7 @@ class EvaluationProvider(Enum):
     OPENAI = 1
     CEREBRAS = 2
     MAITAI = 3
+    SAMBANOVA = 4
 
 
 class AzureParams(BaseModel):
@@ -36,7 +37,7 @@ class ChatCompletionAuth(BaseModel):
     anthropic_api_key: Optional[str] = ""
     cerebras_api_key: Optional[str] = ""
     azure_api_key: Optional[str] = ""
-
+    sambanova_api_key: Optional[str] = ""
 
 class Property(BaseModel):
     type: Union[str, List[str]] = ""
@@ -86,15 +87,7 @@ class ResponseFormat(BaseModel):
 
 
 class StreamOptions(BaseModel):
-    include_usage: bool
-
-
-class ChatMessage(BaseModel):
-    role: str
-    content: Optional[str] = ""
-    name: Optional[str] = None
-    tool_calls: Optional[List["ToolCall"]] = Field(default_factory=list)
-    tool_call_id: Optional[str] = None
+    include_usage: bool = False
 
 
 class ToolCallFunction(BaseModel):
@@ -109,6 +102,14 @@ class ToolCall(BaseModel):
     index: Optional[int] = None
 
 
+class ChatMessage(BaseModel):
+    role: str
+    content: Optional[str] = ""
+    name: Optional[str] = None
+    tool_calls: Optional[List[ToolCall]] = None
+    tool_call_id: Optional[str] = None
+
+
 class ToolChoice(BaseModel):
     type: str
     function: Optional[Function] = None
@@ -118,9 +119,9 @@ class ChatCompletionParams(BaseModel):
     messages: List[ChatMessage]
     model: str
     max_tokens: Optional[int] = None
-    response_format: Optional[ResponseFormat] = ResponseFormat(type="text")
-    temperature: Optional[float] = 0
-    top_p: Optional[float] = 1
+    response_format: Optional[ResponseFormat] = None
+    temperature: Optional[float] = None
+    top_p: Optional[float] = None
     stop: Optional[Union[str, List[str]]] = None
     logprobs: Optional[bool] = False
     top_logprobs: Optional[int] = None
@@ -129,13 +130,14 @@ class ChatCompletionParams(BaseModel):
     stream: Optional[bool] = False
     stream_options: Optional[StreamOptions] = None
     logit_bias: Optional[Dict[str, float]] = None
-    presence_penalty: Optional[float] = 0
-    frequency_penalty: Optional[float] = 0
+    presence_penalty: Optional[float] = None
+    frequency_penalty: Optional[float] = None
     user: Optional[str] = None
     tools: Optional[List[Tool]] = []
     tool_choice: Optional[Union[ToolChoice, str]] = None
     parallel_tool_calls: Optional[bool] = False
     extra_headers: Optional[Dict[str, str]] = None
+    reasoning_effort: Optional[str] = None
 
 
 class ChatCompletionRequest(BaseModel):
@@ -177,7 +179,7 @@ class ChatCompletionMessage(BaseModel):
     role: str
     content: Optional[str] = ""
     function_call: Optional[FunctionCall] = None
-    tool_calls: Optional[List[ToolCall]] = Field(default_factory=list)
+    tool_calls: Optional[List[ToolCall]] = None
     refusal: Optional[str] = None
 
 
@@ -237,6 +239,7 @@ class EvaluateResult(BaseModel):
     evaluation_request_id: Optional[str] = None
     request_id: Optional[str] = None
     application_id: Optional[int] = None
+    session_id: Optional[str] = None
 
 
 class EvaluateResponse(BaseModel):
@@ -255,7 +258,7 @@ class EvaluateResponse(BaseModel):
 
 
 class ChatCompletionResponse(BaseModel):
-    id: Optional[str] = None
+    id: Optional[Union[int, str]] = None
     object: str = "chat.completion"
     created: int = 0
     model: str = None
@@ -277,7 +280,7 @@ class ChatStorageRequest(BaseModel):
     chat_completion_request: ChatCompletionRequest
     chat_completion_response: ChatCompletionResponse
     evaluate_request: Optional["EvaluateRequest"] = None
-    timing_metrics: metric.RequestTimingMetric
+    timing_metrics: Optional[metric.RequestTimingMetric] = None
 
 
 class Turn(BaseModel):
@@ -292,7 +295,7 @@ class Turn(BaseModel):
 class ChoiceDelta(BaseModel):
     content: Optional[str] = ""
     role: Optional[str] = None
-    tool_calls: Optional[List[ToolCall]] = Field(default_factory=list)
+    tool_calls: Optional[List[ToolCall]] = None
     function_call: Optional[FunctionCall] = None
     refusal: Optional[str] = None
 
